/*
 * main.c
 *
 * Created: 05.01.2017 16:11:55
 * Author : Thomas
 */ 

#define F_CPU 16000000UL


#include <avr/io.h>
#include <avr/xmega.h>
#include <util/delay.h>

//#include <stdio.h>
//#include <stdbool.h>

#include "HSV2RGB.h"



int main(void)
{
	//// System clock on PORTB PIN5
	//_PROTECTED_WRITE(CLKCTRL_MCLKCTRLA, CLKCTRL_CLKOUT_bm);
	
	// Full speed, prescaler off (16MHz)
	_PROTECTED_WRITE(CLKCTRL_MCLKCTRLB, !CLKCTRL_PEN_bm);
	
	
	
	// USART in master SPI mode

	PORTMUX.CTRLB = PORTMUX_USART0_bm;	// Alternative USART0 output

	VPORTA.DIR |= PIN3_bm; // | PIN1_bm; // 3 XCK, 1 TxD (TxD not necessary / for debugging)


	USART0.BAUD = (F_CPU / 2 / 800000) << 6;	// 800kHz
	USART0.CTRLC = USART_UCPHA_bm | !USART_UDORD_bm | USART_CMODE_MSPI_gc;
	USART0.CTRLB = USART_TXEN_bm;

	
	
	// TCB0 single shot
	TCB0.CTRLB = TCB_CNTMODE_SINGLE_gc | !TCB_ASYNC_bm; // | TCB_CCMPEN_bm;	// TCB_CCMPEN_bm Pin Output Enable not necessary, for debugging (PORTA PIN5)
	TCB0.EVCTRL = !TCB_FILTER_bm | TCB_CAPTEI_bm;
	TCB0.CCMP = 8;	// 875s high at bit == 1	(312.5 + (8+1)*62.5)
	TCB0.CTRLA = TCB_CLKSEL_CLKDIV1_gc | TCB_ENABLE_bm;
		


	// EVENT XCK->TCB0 single shot (timer start 312.5s(5) after XCK pos. edge)
	EVSYS.ASYNCCH0 = EVSYS_ASYNCCH0_PORTA_PIN3_gc;		// USART XCK (producer)
	EVSYS.ASYNCUSER0 = EVSYS_ASYNCUSER0_ASYNCCH0_gc;	// TCB0 single shot (consumer)

		
	
	// CCL

	//      -----   -----
	// XCK  |   |   |   |
	//      -   -----   -----
	//              ---------
	// TxD          |       |
	//      ---------       -
	//        ----    ----
	// TCB    |  |    |  |
	//      ---  ------  ----
	//      ---     ------
	// WS   | |     |    |
	//      - -------    ----
	// s   312/938 875/375
			
	//  TCB TxD XCK     WS
	//  0   0   0       0
	//  0   0   1       1
	//  0   1   0       0
	//  0   1   1       1
	//  1   0   0       0
	//  1   0   1       0
	//  1   1   0       1
	//  1   1   1       1
	
	//VPORTA.DIR |= PIN7_bm;	// LUT1 out (CCL_OUTEN_bm override port function)
	//PORTA_PIN7CTRL = PORT_INVEN_bm; //invert possibility 1

	// INSEL0 = USART0 XCK
	// INSEL1 = USART0 TxD
	// INSEL2 = TCB0 WO0

	CCL.LUT1CTRLB = CCL_INSEL0_USART0_gc | CCL_INSEL1_USART0_gc;
	CCL.LUT1CTRLC = CCL_INSEL2_TCB0_gc;
	CCL.TRUTH1 = 0b11001010; // invert (uint8_t)~0b11001010; possibility 2
	CCL.LUT1CTRLA = CCL_OUTEN_bm | CCL_ENABLE_bm;
	CCL.CTRLA = CCL_ENABLE_bm;

	

	// Prepare effect
	#define LED_COUNT 8
	
	static HSV_t HsvStripe[LED_COUNT];
	static RGB_t RgbStripe[LED_COUNT];
		
		
	for (int i = 0; i < LED_COUNT; i++)
	{
		HsvStripe[i].H = i * -16;
		HsvStripe[i].S = 250;
		HsvStripe[i].V = 32;
	}
		
		

	// Main loop
    while (1) 
    {
		
		for (uint8_t i = 0; i < LED_COUNT; i++)
		{
			USART0.TXDATAL = RgbStripe[i].Green;
			while (!(USART0.STATUS & USART_DREIF_bm)) {}
			USART0.TXDATAL = RgbStripe[i].Red;
			while (!(USART0.STATUS & USART_DREIF_bm)) {}
			USART0.TXDATAL = RgbStripe[i].Blue;
			while (!(USART0.STATUS & USART_DREIF_bm)) {}
		}
		
		
		// Calculate effect
		for (int i = 0; i < LED_COUNT; i++)
		{
			HsvStripe[i].H += 2;
			HSV2RGB(&HsvStripe[i], &RgbStripe[i]);
		}
		
		_delay_ms(20);
    }
}

